/*
**
**   This code is a part of the ETSI Technical Specification 06.77,
**   Digital cellular telecommunications system (Phase 2+);
**   Minimum Performance Requirements for Noise Suppresser Application to the AMR Speech Encoder
**
*/

/*
       =============================================================
       COPYRIGHT NOTE: This source code, and all of its derivations,
       is subject to the "ITU-T SOFTWARE TOOLS' GENERAL PUBLIC LICENSE". 
	   Please have it  read  in    the  distribution  disk,   or  in  
	   the  ITU-T Recommendation G.191 on "SOFTWARE TOOLS FOR SPEECH AND  
	   AUDIO CODING STANDARDS".
       =============================================================
*/



/*
**
**   OBJECTIVE MEASURES FOR CHARACTERISATION OF AMR/NS SOLUTIONS
**
**
**   MODULE:         main.c, main program for the computation of the
**                     objective measures for the characterisation of NS performance
**   ORIGINAL BY:    
**      Beghdad Ayad     Nokia Research Center    Finland
**      Erkki Paajanen   Nokia Research Center    Finland
**   DATE:           19/Jan/2001
**   RELEASE:        1.00
**   FUNCTIONS:      main .............. initialisation of data structures, management of I/O
**                                       files, calling of Process to compute objective
**                                       measures, calculation of statistics on objective
**                                       measures and organising and writing of report
**                   NextFile .......... find and return next file in claimed speech database
**   PROTOTYPES:     in main.c
**
**
**   MODIFICATION HISTORY:
**      15/Jun/00 v0.90 Release for inclusion in ETSI TS 06.77.   <erkki.paajanen@nokia.com>
**      19/Jan/01 v1.00 CR release: Inclusion of "ITU-T General Public License" terms.
**                                                                <erkki.paajanen@nokia.com>
**
**
*/


/*
** Copyright (c) 1999-2001 Nokia Corporation
*/


#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "main.h"


/*---------------------------------------------------------------------------
 *                              Constants
 *--------------------------------------------------------------------------*/
#define FILENAME_SIZE                (29+strlen(DATABASE_ROOT))

 /*---------------------------------------------------------------------------
 *                                Types
 *--------------------------------------------------------------------------*/
typedef struct {
   NoiseType  noise;
   int        total;
} StatNoise;


/*---------------------------------------------------------------------------
 *                                Functions
 *--------------------------------------------------------------------------*/
char *NextFile(FileDesc *);

extern char *cond_name[];


/*---------------------------------------------------------------------------
 *                                Data
 *--------------------------------------------------------------------------*/
static char *gender[]={"f", "m"};

static char CleanSpeechName[200];
static char NoisySpeechName[200];
static char AmrOutputName[200];
static char AmrNsOutputName[200];


typedef enum {
   TEST = 1,
   READ_AND_CALCULATE,
   PROCESS_AND_DELETE,
   PROCESS_AND_STORE
} RunMode;

/*---------------------------------------------------------------------------
 
        int main(int argc, char *argv[])
            ====

        Description:
        ~~~~~~~~~~~~

        Initialises all data structures needed for the NS objective measures,
        handles speech file control, calls 'Process' to optionally process the
        AMR speech codec and the AMR/NS solution on the speech database and
        always to compute the NS objective measures on the speech database, and
        organises and produces a report on the objective measure results.

        Arguments:
        ~~~~~~~~~~
        Name:        I/0: Type:  Use:
        argc          I   int    Number of arguments (including program name)
        argv          I   char * Table of main program argument strings

        Return value:
        ~~~~~~~~~~~~~
        Returns zero on successful exit and a non-zero value on unsuccessful exit.

        Functions used:        NextFile
        ~~~~~~~~~~~~~~~        Process

        Prototype:             none
        ~~~~~~~~~~

        Author:                Beghdad Ayad     Nokia Research Center   Finland
        ~~~~~~~                Erkki Paajanen   Nokia Research Center   Finland
                               <erkki.paajanen@nokia.com>

        Modification history:
        ~~~~~~~~~~~~~~~~~~~~~
        Date         Version   Description
        07/Jun/00    0.10      Preliminary version for review in SMG11
                               <erkki.paajanen@nokia.com>
        19/Jan/00    1.00      CR release for TS 06.77.
                               <erkki.paajanen@nokia.com>

 *--------------------------------------------------------------------------*/
/*
** Copyright (c) 1999-2001 Nokia Corporation
*/
int main(int argc, char *argv[])
{
   int n, N;
   Report* report;
   FILE  *Report_File;
   int  cdt;
   int  cdt0 = -1;
   RunMode  run_mode;
   char ans;

   /* Counters for statistics */
   /* Per test condition  */
   int      nbFile[NB_CDT];
   int      NbFrFrames[NB_CDT];
   int      KsphFrames[NB_CDT];
   int      KspmFrames[NB_CDT];
   int      KsplFrames[NB_CDT];
   int      KnseFrames[NB_CDT];
   int      KtotFrames[NB_CDT];

   float    SNR_imp_h[NB_CDT];
   float    SNR_imp_m[NB_CDT];
   float    SNR_imp_l[NB_CDT];
   float    SNR_imp[NB_CDT];
   float    NOISE_PLR[NB_CDT];

   float    SNR_imp_h_2[NB_CDT];
   float    SNR_imp_m_2[NB_CDT];
   float    SNR_imp_l_2[NB_CDT];
   float    SNR_imp_2[NB_CDT];
   float    NOISE_PLR_2[NB_CDT];

   int      nbFile_car    = 0;
   float    SNR_imp_h_car = 0.f;
   float    SNR_imp_m_car = 0.f;
   float    SNR_imp_l_car = 0.f;
   float    SNR_imp_car   = 0.f;
   float    NOISE_PLR_car = 0.f;

   float    SNR_imp_h_2_car = 0.f;
   float    SNR_imp_m_2_car = 0.f;
   float    SNR_imp_l_2_car = 0.f;
   float    SNR_imp_2_car   = 0.f;
   float    NOISE_PLR_2_car = 0.f;

   int      nbFile_str    = 0;
   float    SNR_imp_h_str = 0.f;
   float    SNR_imp_m_str = 0.f;
   float    SNR_imp_l_str = 0.f;
   float    SNR_imp_str   = 0.f;
   float    NOISE_PLR_str = 0.f;

   float    SNR_imp_h_2_str = 0.f;
   float    SNR_imp_m_2_str = 0.f;
   float    SNR_imp_l_2_str = 0.f;
   float    SNR_imp_2_str   = 0.f;
   float    NOISE_PLR_2_str = 0.f;

   int      nbFile_ave    = 0;
   float    SNR_imp_h_ave = 0.f;
   float    SNR_imp_m_ave = 0.f;
   float    SNR_imp_l_ave = 0.f;
   float    SNR_imp_ave   = 0.f;
   float    NOISE_PLR_ave = 0.f;

   float    SNR_imp_h_2_ave = 0.f;
   float    SNR_imp_m_2_ave = 0.f;
   float    SNR_imp_l_2_ave = 0.f;
   float    SNR_imp_2_ave   = 0.f;
   float    NOISE_PLR_2_ave = 0.f;



   /* total */
   int      Total_NbFr;
   int      Total_File;
   int      Total_Frames;
   int      Total_sph;
   int      Total_spm;
   int      Total_spl;
   int      Total_nse;

   int      Total_NbFr_car;
   int      Total_Frames_car;
   int      Total_sph_car;
   int      Total_spm_car;
   int      Total_spl_car;
   int      Total_nse_car;

   int      Total_NbFr_str;
   int      Total_Frames_str;
   int      Total_sph_str;
   int      Total_spm_str;
   int      Total_spl_str;
   int      Total_nse_str;

   int      Total_NbFr_ave;
   int      Total_Frames_ave;
   int      Total_sph_ave;
   int      Total_spm_ave;
   int      Total_spl_ave;
   int      Total_nse_ave;

   int      NumbFrames;

   /* Mean */
   float    SNRI_h_per_cond[NB_CDT];
   float    SNRI_m_per_cond[NB_CDT];
   float    SNRI_l_per_cond[NB_CDT];
   float    SNRI_per_cond[NB_CDT];
   float    NPLR_per_cond[NB_CDT];

   float    SNRI_h_aggr_car;
   float    SNRI_m_aggr_car;
   float    SNRI_l_aggr_car;
   float    SNRI_aggr_car;
   float    NPLR_aggr_car;

   float    SNRI_h_aggr_str;
   float    SNRI_m_aggr_str;
   float    SNRI_l_aggr_str;
   float    SNRI_aggr_str;
   float    NPLR_aggr_str;

   float    SNRI_h_aggr_ave;
   float    SNRI_m_aggr_ave;
   float    SNRI_l_aggr_ave;
   float    SNRI_aggr_ave;
   float    NPLR_aggr_ave;
  

   /* Standard deviation */
   float    SNRI_h_STD_per_cond[NB_CDT];
   float    SNRI_m_STD_per_cond[NB_CDT];
   float    SNRI_l_STD_per_cond[NB_CDT];
   float    SNRI_STD_per_cond[NB_CDT];
   float    NPLR_STD_per_cond[NB_CDT];

   float    SNRI_h_STD_aggr_car;
   float    SNRI_m_STD_aggr_car;
   float    SNRI_l_STD_aggr_car;
   float    SNRI_STD_aggr_car;
   float    NPLR_STD_aggr_car;

   float    SNRI_h_STD_aggr_str;
   float    SNRI_m_STD_aggr_str;
   float    SNRI_l_STD_aggr_str;
   float    SNRI_STD_aggr_str;
   float    NPLR_STD_aggr_str;

   float    SNRI_h_STD_aggr_ave;
   float    SNRI_m_STD_aggr_ave;
   float    SNRI_l_STD_aggr_ave;
   float    SNRI_STD_aggr_ave;
   float    NPLR_STD_aggr_ave;


   float    Mu, delta2, K095;


   if (argc==1 || argv[1][0] == '-' && (argv[1][1] == 'h' || argv[1][1] == 'H'))
      goto USAGE;

   run_mode = atoi(argv[1]);
   if (run_mode < 1 || run_mode > 4)
      goto USAGE;
   if (run_mode == 4)
   {
      fprintf(stdout, "\nThis option will overwrite your possible earlier AMR & AMR/NS processing results!\n");
      fprintf(stdout, "\nDo you want to proceed (y/n)?\n\n");
      ans = fgetc(stdin);
      if (!(ans == 'y' || ans == 'Y'))
         goto USAGE;
   }


   /* Open report file */

   if (argc>2)
   {
     Report_File = fopen(argv[2], "wt");
     if (NULL == Report_File)
     {
       printf("Unable to open Report File %s \n", argv[2]);
       exit(1);
     }
   }
   else
   {
     Report_File = fopen(REPORT_FILENAME,"wt");
     if(NULL == Report_File)
     {
       printf("Unable to open Report File %s \n", REPORT_FILENAME);
       exit(1);
     }
   }

   
   for(cdt=0;cdt<NB_CDT;cdt++) {

      nbFile[cdt]       = 0;
      NbFrFrames[cdt]   = 0;
      KtotFrames[cdt]   = 0;
      KsphFrames[cdt]   = 0;
      KspmFrames[cdt]   = 0;
      KsplFrames[cdt]   = 0;
      KnseFrames[cdt]   = 0;

      SNR_imp_h[cdt]    = 0.f;
      SNR_imp_m[cdt]    = 0.f;
      SNR_imp_l[cdt]    = 0.f;
      SNR_imp[cdt]      = 0.f;
      NOISE_PLR[cdt]    = 0.f;

      SNR_imp_h_2[cdt]  = 0.f;
      SNR_imp_m_2[cdt]  = 0.f;
      SNR_imp_l_2[cdt]  = 0.f;
      SNR_imp_2[cdt]    = 0.f;
      NOISE_PLR_2[cdt]  = 0.f;
      
      SNRI_h_STD_per_cond[cdt] = 0.f;
      SNRI_m_STD_per_cond[cdt] = 0.f;
      SNRI_l_STD_per_cond[cdt] = 0.f;
      SNRI_STD_per_cond[cdt]   = 0.f;
      NPLR_STD_per_cond[cdt]   = 0.f;
   } /* for(cdt=0... */


   /* Print used definitions ...*/
   fprintf(Report_File, "\n############################################################### \n");
   fprintf(Report_File,   "#       REPORT ON OBJECTIVE MEASURES ON NS PERFORMANCE        # \n");
   fprintf(Report_File,   "############################################################### \n\n");
   fprintf(Report_File, "\nDEFINITIONS:\n");
   fprintf(Report_File, "Filename   : path and filename of the preprocessed file\n");
   fprintf(Report_File, "|SNRI      : signal-to-noise ratio (SNR) improvement\n");
   fprintf(Report_File, "|Ktot      : total number of frames classified as speech\n");
   fprintf(Report_File, "|NPLR      : NPLR noise power level reduction\n");
   fprintf(Report_File, "|Knse      : total number of frames containing noise only\n");
   fprintf(Report_File, "|SNRI_h    : SNRI for high power frames\n");
   fprintf(Report_File, "|Ksph      : total number of frames containing speech of a high power\n");
   fprintf(Report_File, "|SNRI_m    : SNRI for medium power frames\n");
   fprintf(Report_File, "|Kspm      : total number of frames containing speech of a medium power\n");
   fprintf(Report_File, "|SNRI_l    : SNRI for lower frames \n");
   fprintf(Report_File, "|Kspl      : total number of frames containing speech of a low power\n");
   fprintf(Report_File, "|+/-       : refers to the 95%% confidence interval\n");

   fflush(Report_File);



   /* For every preprocessed file */

   for(n = 0; files[n].noise >= 0; n++)
   {
      char *fileName;
      
      
      /* Compute next filename */
      fileName = NextFile(&files[n]);
      cdt = (int)(files[n].noise);

      if(cdt>cdt0)
      {
        fprintf(Report_File, "\n---------------------------------------------------------------\n");
        fprintf(Report_File, "%s \n", cond_name[cdt]);
        fprintf(Report_File, "...............................................................\n");
        /* ... and column titles */
        fprintf(Report_File, "%-*s   %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s %6s\n",FILENAME_SIZE,
           "Filename","Ksph","SNRI_h","Kspm","SNRI_m","Kspl","SNRI_l","Ktot","SNRI","Knse","NPLR");

        fflush(Report_File);
   
        cdt0 = cdt;
      } /* if(cdt>cdt0) */
   

      while(fileName)
      {

         /* Compute SNRI_h,l,m and NPLR values for this file */

         strcpy(CleanSpeechName, fileName);
         strcat(CleanSpeechName, CLEAN_EXT); 

         strcpy(NoisySpeechName, fileName);
         strcat(NoisySpeechName, NOISY_EXT);

         if (run_mode == 2 || run_mode == 4) /* Use outputs processed earlier or process and store outputs */
         {

            strcpy(AmrOutputName, fileName);
            strcat(AmrOutputName, AMR_EXT);

            strcpy(AmrNsOutputName, fileName);
            strcat(AmrNsOutputName, AMRNS_EXT);

         }

         else if (run_mode == 3) /* Process and delete outputs */
         {

            strcpy(AmrOutputName, "AMROut.tmp");
            strcpy(AmrNsOutputName, "AMRNSOut.tmp");

         }

         
         report = Process(run_mode, CleanSpeechName, NoisySpeechName,
            AmrOutputName, AmrNsOutputName, files[n].mode);

          /* store values for statistics */
      
         nbFile[cdt]++ ;

         NbFrFrames[cdt]   += report->NbFr;
         
         KsphFrames[cdt]   += report->Ksph;
         KspmFrames[cdt]   += report->Kspm;
         KsplFrames[cdt]   += report->Kspl;
         KnseFrames[cdt]   += report->Knse;

         SNR_imp_h[cdt]    += (report->SNRI_h);
         SNR_imp_m[cdt]    += (report->SNRI_m);
         SNR_imp_l[cdt]    += (report->SNRI_l);
         
         NOISE_PLR[cdt]    += (report->NPLR);


         SNR_imp_h_2[cdt]  += (report->SNRI_h)*(report->SNRI_h);
         SNR_imp_m_2[cdt]  += (report->SNRI_m)*(report->SNRI_m);
         SNR_imp_l_2[cdt]  += (report->SNRI_l)*(report->SNRI_l);
         
         NOISE_PLR_2[cdt]  += (report->NPLR)*(report->NPLR);

         KtotFrames[cdt]   += report->Ktot;
         SNR_imp[cdt]      += (report->SNRI);
         SNR_imp_2[cdt]    += (report->SNRI)*(report->SNRI);

         /* Current file objective measures values */

         fprintf(Report_File, "%*s ",FILENAME_SIZE,CleanSpeechName);
         fprintf(Report_File, "  %6d",report->Ksph);
         fprintf(Report_File, "  %5.2f",report->SNRI_h);
         fprintf(Report_File, "  %6d ",report->Kspm);
         fprintf(Report_File, "  %5.2f",report->SNRI_m);
         fprintf(Report_File, "  %6d ",report->Kspl);
         fprintf(Report_File, "  %5.2f",report->SNRI_l);
         fprintf(Report_File, "  %6d ",report->Ktot);
         fprintf(Report_File, "  %5.2f",report->SNRI);
         fprintf(Report_File, "  %6d ",report->Knse);
         fprintf(Report_File, "  %5.2f \n",report->NPLR);

         fflush(Report_File);
         
         fileName = NextFile(0);
      } /* while(fileName) */
   } /* for(n=0... */

   /* Compute statistics */
   /* Compute totals */
   Total_File    = 0;
   Total_NbFr    = 0;
   Total_Frames  = 0;
   Total_sph     = 0;
   Total_spm     = 0;
   Total_spl     = 0;
   Total_nse     = 0; 

   for(cdt=0;cdt<NB_CDT;cdt++)
   {
      Total_File    += nbFile[cdt];
      Total_NbFr    += NbFrFrames[cdt];
      Total_Frames  += KtotFrames[cdt];
      Total_sph     += KsphFrames[cdt];
      Total_spm     += KspmFrames[cdt];
      Total_spl     += KsplFrames[cdt];
      Total_nse     += KnseFrames[cdt];

      if(cdt==LAST_CAR) 
      {   
          Total_NbFr_car    = Total_NbFr;
          Total_Frames_car  = Total_Frames;
          Total_sph_car     = Total_sph;
          Total_spm_car     = Total_spm;
          Total_spl_car     = Total_spl;
          Total_nse_car     = Total_nse;
      }

      if(cdt==LAST_STREET) 
      {   
          Total_NbFr_str    = Total_NbFr - Total_NbFr_car;
          Total_Frames_str  = Total_Frames - Total_Frames_car;
          Total_sph_str     = Total_sph - Total_sph_car;
          Total_spm_str     = Total_spm - Total_spm_car;
          Total_spl_str     = Total_spl - Total_spl_car;
          Total_nse_str     = Total_nse - Total_nse_car;
      }

      if(cdt==LAST_AGGREGATE) 
      {   
          Total_NbFr_ave    = Total_NbFr;
          Total_Frames_ave  = Total_Frames;
          Total_sph_ave     = Total_sph;
          Total_spm_ave     = Total_spm;
          Total_spl_ave     = Total_spl;
          Total_nse_ave     = Total_nse;
      }
   } /* for(cdt=0... */


   /* aggregate measure for car noise test conditions */
   for(cdt=0;cdt<=LAST_CAR;cdt++)
   {
       nbFile_car       += nbFile[cdt];
       SNR_imp_h_car    += SNR_imp_h[cdt];
       SNR_imp_m_car    += SNR_imp_m[cdt];
       SNR_imp_l_car    += SNR_imp_l[cdt];
         
       NOISE_PLR_car    += NOISE_PLR[cdt];

       SNR_imp_h_2_car  += SNR_imp_h_2[cdt];
       SNR_imp_m_2_car  += SNR_imp_m_2[cdt];
       SNR_imp_l_2_car  += SNR_imp_l_2[cdt];
         
       NOISE_PLR_2_car  += NOISE_PLR_2[cdt];

       SNR_imp_car      += SNR_imp[cdt];
       SNR_imp_2_car    += SNR_imp_2[cdt];
   }

   /* aggregate measure for street noise test conditions */
   for(cdt=LAST_CAR;cdt<=LAST_STREET;cdt++)
   {
       nbFile_str       += nbFile[cdt];
       SNR_imp_h_str    += SNR_imp_h[cdt];
       SNR_imp_m_str    += SNR_imp_m[cdt];
       SNR_imp_l_str    += SNR_imp_l[cdt];
         
       NOISE_PLR_str    += NOISE_PLR[cdt];

       SNR_imp_h_2_str  += SNR_imp_h_2[cdt];
       SNR_imp_m_2_str  += SNR_imp_m_2[cdt];
       SNR_imp_l_2_str  += SNR_imp_l_2[cdt];
         
       NOISE_PLR_2_str  += NOISE_PLR_2[cdt];

       SNR_imp_str      += SNR_imp[cdt];
       SNR_imp_2_str    += SNR_imp_2[cdt];
   }

   /* aggregate measure for car and street noise */
   for(cdt=0;cdt<=LAST_AGGREGATE;cdt++)
   {
       nbFile_ave       += nbFile[cdt];
       SNR_imp_h_ave    += SNR_imp_h[cdt];
       SNR_imp_m_ave    += SNR_imp_m[cdt];
       SNR_imp_l_ave    += SNR_imp_l[cdt];
         
       NOISE_PLR_ave    += NOISE_PLR[cdt];

       SNR_imp_h_2_ave  += SNR_imp_h_2[cdt];
       SNR_imp_m_2_ave  += SNR_imp_m_2[cdt];
       SNR_imp_l_2_ave  += SNR_imp_l_2[cdt];
         
       NOISE_PLR_2_ave  += NOISE_PLR_2[cdt];

       SNR_imp_ave      += SNR_imp[cdt];
       SNR_imp_2_ave    += SNR_imp_2[cdt];
   }


   /* compute mean and standard deviation */
   /* Per test condition types */

   for(cdt=0;cdt<NB_CDT;cdt++)
   {

      if(KtotFrames[cdt] != 0)
      {
         N      = nbFile[cdt];

         Mu     = SNRI_h_per_cond[cdt] = SNR_imp_h[cdt]/N;
         delta2 = SNR_imp_h_2[cdt]/N - Mu*Mu;
         SNRI_h_STD_per_cond[cdt] = (float)sqrt(delta2*N/(N-1));

         Mu     = SNRI_m_per_cond[cdt] = SNR_imp_m[cdt]/N;
         delta2 = SNR_imp_m_2[cdt]/N - Mu*Mu;
         SNRI_m_STD_per_cond[cdt] = (float)sqrt(delta2*N/(N-1));

         Mu     = SNRI_l_per_cond[cdt] = SNR_imp_l[cdt]/N;
         delta2 = SNR_imp_l_2[cdt]/N - Mu*Mu;
         SNRI_l_STD_per_cond[cdt] = (float)sqrt(delta2*N/(N-1));
         
         Mu     = SNRI_per_cond[cdt]   = SNR_imp[cdt]/N;
         delta2 = SNR_imp_2[cdt]/N - Mu*Mu; 
         SNRI_STD_per_cond[cdt] = (float)sqrt(delta2*N/(N-1));

         Mu     = NPLR_per_cond[cdt]   = NOISE_PLR[cdt]/N;
         delta2 = NOISE_PLR_2[cdt]/N - Mu*Mu;
         NPLR_STD_per_cond[cdt] = (float)sqrt(delta2*N/(N-1));
      }
      else
      {
         SNRI_h_per_cond[cdt] = -100.;
         SNRI_m_per_cond[cdt] = -100.;
         SNRI_l_per_cond[cdt] = -100.;
         SNRI_per_cond[cdt]   = -100.;
         NPLR_per_cond[cdt]   =  100.;     
      }
   }

   /* mean and standard deviation for car noise */

   N      = nbFile_car;
   Mu     = SNRI_h_aggr_car = SNR_imp_h_car/N;
   delta2 = SNR_imp_h_2_car/N - Mu*Mu;
   SNRI_h_STD_aggr_car = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_m_aggr_car = SNR_imp_m_car/N;
   delta2 = SNR_imp_m_2_car/N - Mu*Mu;
   SNRI_m_STD_aggr_car = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_l_aggr_car = SNR_imp_l_car/N;
   delta2 = SNR_imp_l_2_car/N - Mu*Mu;
   SNRI_l_STD_aggr_car = (float)sqrt(delta2*N/(N-1));
         
   Mu     = SNRI_aggr_car   = SNR_imp_car/(float)N;
   delta2 = SNR_imp_2_car/(float)N - Mu*Mu;
   SNRI_STD_aggr_car = (float)sqrt(delta2*N/(N-1));      

   Mu     = NPLR_aggr_car   = NOISE_PLR_car/N;
   delta2 = NOISE_PLR_2_car/N - Mu*Mu;
   NPLR_STD_aggr_car = (float)sqrt(delta2*N/(N-1));

   /* mean and standard deviation for street noise */

   N      = nbFile_str;
   Mu     = SNRI_h_aggr_str = SNR_imp_h_str/N;
   delta2 = SNR_imp_h_2_str/N - Mu*Mu;
   SNRI_h_STD_aggr_str = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_m_aggr_str = SNR_imp_m_str/N;
   delta2 = SNR_imp_m_2_str/N - Mu*Mu;
   SNRI_m_STD_aggr_str = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_l_aggr_str = SNR_imp_l_str/N;
   delta2 = SNR_imp_l_2_str/N - Mu*Mu;
   SNRI_l_STD_aggr_str = (float)sqrt(delta2*N/(N-1));
         
   Mu     = SNRI_aggr_str   = SNR_imp_str/(float)N;
   delta2 = SNR_imp_2_str/(float)N - Mu*Mu;
   SNRI_STD_aggr_str = (float)sqrt(delta2*N/(N-1));      

   Mu     = NPLR_aggr_str   = NOISE_PLR_str/N;
   delta2 = NOISE_PLR_2_str/N - Mu*Mu;
   NPLR_STD_aggr_str = (float)sqrt(delta2*N/(N-1));

   /* mean and standard deviation for aggregate measure on car and street noise conditions */

   N      = nbFile_ave;
   Mu     = SNRI_h_aggr_ave = SNR_imp_h_ave/N;
   delta2 = SNR_imp_h_2_ave/N - Mu*Mu;
   SNRI_h_STD_aggr_ave = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_m_aggr_ave = SNR_imp_m_ave/N;
   delta2 = SNR_imp_m_2_ave/N - Mu*Mu;
   SNRI_m_STD_aggr_ave = (float)sqrt(delta2*N/(N-1));

   Mu     = SNRI_l_aggr_ave = SNR_imp_l_ave/N;
   delta2 = SNR_imp_l_2_ave/N - Mu*Mu;
   SNRI_l_STD_aggr_ave = (float)sqrt(delta2*N/(N-1));
         
   Mu     = SNRI_aggr_ave   = SNR_imp_ave/(float)N;
   delta2 = SNR_imp_2_ave/(float)N - Mu*Mu;
   SNRI_STD_aggr_ave = (float)sqrt(delta2*N/(N-1));      

   Mu     = NPLR_aggr_ave   = NOISE_PLR_ave/N;
   delta2 = NOISE_PLR_2_ave/N - Mu*Mu;
   NPLR_STD_aggr_ave = (float)sqrt(delta2*N/(N-1));


   /* Report statistics */

   NumbFrames    = Total_Frames + Total_nse;

   fprintf(Report_File, "\n\n\n");
   fprintf(Report_File,   "###############################################################\n");
   fprintf(Report_File,   "###############################################################\n");
   fprintf(Report_File,   "##                                                           ##\n");
   fprintf(Report_File,   "##                  OBJECTIVE MEASURE RESULTS                ##\n");
   fprintf(Report_File,   "##                                                           ##\n");
   fprintf(Report_File,   "###############################################################\n");
   fprintf(Report_File,   "###############################################################\n\n");
  
   fprintf(Report_File, "%10d total processed files \n",Total_File);
   fprintf(Report_File, "%10d total frames \n\n",Total_NbFr);
   fprintf(Report_File, " ------ Characteristics  of the database ------ \n");
   fprintf(Report_File, "%4.2f%% of frames used for classification \n\n",100*(float)NumbFrames/Total_NbFr);

   fprintf(Report_File, " ---- Classified Frames ---- \n");
   fprintf(Report_File, "%4.2f%% of only noise power frames \n",100*(float)Total_nse/NumbFrames);
   fprintf(Report_File, "%4.2f%% of speech frames \n\n",100*(float)Total_Frames/NumbFrames);
   
   fprintf(Report_File, " --- Speech Classes --- \n");
   fprintf(Report_File, "%4.2f%% of high power frames \n",100*(float)Total_sph/Total_Frames);
   fprintf(Report_File, "%4.2f%% of medium power frames \n",100*(float)Total_spm/Total_Frames);
   fprintf(Report_File, "%4.2f%% of low power frames \n\n\n",100*(float)Total_spl/Total_Frames);

   fprintf(Report_File, "\n############################################################### \n");
   fprintf(Report_File,   "#                   RESULTS PER TEST CONDITION                # \n");
   fprintf(Report_File,   "############################################################### \n\n");

   fflush(Report_File);


   for(cdt=0;cdt<NB_CDT;cdt++) {

      NumbFrames = KtotFrames[cdt]+KnseFrames[cdt];

      fprintf(Report_File, "--------------------------------------------------------------- \n");
      fprintf(Report_File, "%s \n", cond_name[cdt]);
      fprintf(Report_File, "''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' \n");
      fprintf(Report_File, "%10d files \n",nbFile[cdt]);
      fprintf(Report_File, "%10d frames \n\n",NbFrFrames[cdt]);
      fprintf(Report_File, " ------ Characteristics of the database ------ \n");
      fprintf(Report_File, "%4.2f%% of frames used for classification \n\n",100*(float)NumbFrames/NbFrFrames[cdt]);


      K095   = 1.96f/(float)sqrt((double)nbFile[cdt]); /* Factor for a 95% confidence Inteval */
   
      fprintf(Report_File, " ---- Classified Frames ---- \n");
      fprintf(Report_File, "%4.2f%% of only noise frames  \tNPLR   = %5.2f\t+/- %4.2f \n",100*(float)KnseFrames[cdt]/NumbFrames,NPLR_per_cond[cdt],NPLR_STD_per_cond[cdt]*K095);
      fprintf(Report_File, "%4.2f%% of speech frames      \tSNRI   = %5.2f\t+/- %4.2f \n\n",100*(float)KtotFrames[cdt]/NumbFrames,   SNRI_per_cond[cdt],(float)SNRI_STD_per_cond[cdt]*K095);
   
      fprintf(Report_File, " --- Speech Classes --- \n");
      fprintf(Report_File, "%4.2f%% of high power frames  \tSNRI_h = %5.2f\t+/- %4.2f \n",100*(float)KsphFrames[cdt]/KtotFrames[cdt],  SNRI_h_per_cond[cdt],SNRI_h_STD_per_cond[cdt]*K095);
      fprintf(Report_File, "%4.2f%% of medium power frames\tSNRI_m = %5.2f\t+/- %4.2f \n",100*(float)KspmFrames[cdt]/KtotFrames[cdt],  SNRI_m_per_cond[cdt],SNRI_m_STD_per_cond[cdt]*K095);
      fprintf(Report_File, "%4.2f%% of low power frames   \tSNRI_l = %5.2f\t+/- %4.2f \n\n",100*(float)KsplFrames[cdt]/KtotFrames[cdt],  SNRI_l_per_cond[cdt],SNRI_l_STD_per_cond[cdt]*K095);

      fflush(Report_File);
   
   } /* for(cdt=0... */

   
   NumbFrames = Total_Frames_car + Total_nse_car ;
   K095   = 1.96f/(float)sqrt((double)nbFile_car);

   fprintf(Report_File, "\n################################################################\n");
   fprintf(Report_File,   "#  AGGREGATE MEASURE FOR 4 CAR NOISE CONDITIONS (1a,1b,2a,2b)  #\n");
   fprintf(Report_File,   "################################################################\n");
   fprintf(Report_File, "%10d files  \n",nbFile_car);
   fprintf(Report_File, "%10d frames \n\n",Total_NbFr_car);
   fprintf(Report_File, " ------ Characteristics of the database ------ \n");
   fprintf(Report_File, "%4.2f%% of frames used for classification \n\n",100*(float)NumbFrames/Total_NbFr_car);

   fprintf(Report_File, " ---- Classified Frames ---- \n");
   fprintf(Report_File, "%4.2f%% of only noise frames  \tNPLR   = %5.2f\t+/- %4.2f \n",100*(float)Total_nse_car/NumbFrames,NPLR_aggr_car,NPLR_STD_aggr_car*K095);
   fprintf(Report_File, "%4.2f%% of speech frames      \tSNRI   = %5.2f\t+/- %4.2f \n\n",100*(float)Total_Frames_car/NumbFrames, SNRI_aggr_car,(float)SNRI_STD_aggr_car*K095);

   fprintf(Report_File, " --- Speech Classes ---\n");
   fprintf(Report_File, "%4.2f%% of high power frame   \tSNRI_h = %5.2f\t+/- %4.2f \n",100*(float)Total_sph_car/Total_Frames_car,  SNRI_h_aggr_car,SNRI_h_STD_aggr_car*K095);
   fprintf(Report_File, "%4.2f%% of medium power frames\tSNRI_m = %5.2f\t+/- %4.2f \n",100*(float)Total_spm_car/Total_Frames_car,  SNRI_m_aggr_car,SNRI_m_STD_aggr_car*K095);
   fprintf(Report_File, "%4.2f%% of low power frames   \tSNRI_l = %5.2f\t+/- %4.2f \n\n\n",100*(float)Total_spl_car/Total_Frames_car,SNRI_l_aggr_car,SNRI_l_STD_aggr_car*K095);
  
   fflush(Report_File);
   

   NumbFrames = Total_Frames_str + Total_nse_str ;
   K095   = 1.96f/(float)sqrt((double)nbFile_str);

   fprintf(Report_File, "\n###################################################################\n");
   fprintf(Report_File,   "#  AGGREGATE MEASURE FOR 4 STREET NOISE CONDITIONS (3a,3b,4a,4b)  #\n");
   fprintf(Report_File,   "###################################################################\n");
   fprintf(Report_File, "%10d files  \n",nbFile_str);
   fprintf(Report_File, "%10d frames \n\n",Total_NbFr_str);
   fprintf(Report_File, " ------ Characteristics of the database ------ \n");
   fprintf(Report_File, "%4.2f%% of frames used for classification \n\n",100*(float)NumbFrames/Total_NbFr_str);

   fprintf(Report_File, " ---- Classified Frames ---- \n");
   fprintf(Report_File, "%4.2f%% of only noise frames  \tNPLR   = %5.2f\t+/- %4.2f \n",100*(float)Total_nse_str/NumbFrames,NPLR_aggr_str,NPLR_STD_aggr_str*K095);
   fprintf(Report_File, "%4.2f%% of speech frames      \tSNRI   = %5.2f\t+/- %4.2f \n\n",100*(float)Total_Frames_str/NumbFrames, SNRI_aggr_str,(float)SNRI_STD_aggr_str*K095);

   fprintf(Report_File, " --- Speech Classes ---\n");
   fprintf(Report_File, "%4.2f%% of high power frame   \tSNRI_h = %5.2f\t+/- %4.2f \n",100*(float)Total_sph_str/Total_Frames_str,  SNRI_h_aggr_str,SNRI_h_STD_aggr_str*K095);
   fprintf(Report_File, "%4.2f%% of medium power frames\tSNRI_m = %5.2f\t+/- %4.2f \n",100*(float)Total_spm_str/Total_Frames_str,  SNRI_m_aggr_str,SNRI_m_STD_aggr_str*K095);
   fprintf(Report_File, "%4.2f%% of low power frames   \tSNRI_l = %5.2f\t+/- %4.2f \n\n\n",100*(float)Total_spl_str/Total_Frames_str,SNRI_l_aggr_str,SNRI_l_STD_aggr_str*K095);

   fflush(Report_File);


   NumbFrames = Total_Frames_ave + Total_nse_ave ;
   K095   = 1.96f/(float)sqrt((double)nbFile_ave);

   fprintf(Report_File, "\n##################################################################\n");
   fprintf(Report_File,   "#  AGGREGATE MEASURE FOR 8 CONDITIONS (1a,1b,2a,2b,3a,3b,4a,4b)  #\n");
   fprintf(Report_File,   "##################################################################\n");
   fprintf(Report_File, "%10d files  \n",nbFile_ave);
   fprintf(Report_File, "%10d frames \n\n",Total_NbFr_ave);
   fprintf(Report_File, " ------ Characteristics of the database ------ \n");
   fprintf(Report_File, "%4.2f%% of frames used for classification \n\n",100*(float)NumbFrames/Total_NbFr_ave);

   fprintf(Report_File, " ---- Classified Frames ---- \n");
   fprintf(Report_File, "%4.2f%% of only noise frames  \tNPLR   = %5.2f\t+/- %4.2f \n",100*(float)Total_nse_ave/NumbFrames,NPLR_aggr_ave,NPLR_STD_aggr_ave*K095);
   fprintf(Report_File, "%4.2f%% of speech frames      \tSNRI   = %5.2f\t+/- %4.2f \n\n",100*(float)Total_Frames_ave/NumbFrames, SNRI_aggr_ave,(float)SNRI_STD_aggr_ave*K095);

   fprintf(Report_File, " --- Speech Classes ---\n");
   fprintf(Report_File, "%4.2f%% of high power frame   \tSNRI_h = %5.2f\t+/- %4.2f \n",100*(float)Total_sph_ave/Total_Frames_ave,  SNRI_h_aggr_ave,SNRI_h_STD_aggr_ave*K095);
   fprintf(Report_File, "%4.2f%% of medium power frames\tSNRI_m = %5.2f\t+/- %4.2f \n",100*(float)Total_spm_ave/Total_Frames_ave,  SNRI_m_aggr_ave,SNRI_m_STD_aggr_ave*K095);
   fprintf(Report_File, "%4.2f%% of low power frames   \tSNRI_l = %5.2f\t+/- %4.2f \n\n\n",100*(float)Total_spl_ave/Total_Frames_ave,SNRI_l_aggr_ave,SNRI_l_STD_aggr_ave*K095);

   fflush(Report_File);
   
   
   /* Close report file */
   fclose (Report_File);
   return(0);

USAGE:
   fprintf(stderr, "\nSNRINPLR: Objective measures for NS performance characterisation\n");
   fprintf(stderr, "\nUsage: %s RunMode [Report_File_Name]\n\n", argv[0]);
   fprintf(stderr, "  RunMode:    1  Test existence of files and produce dummy report\n");
   fprintf(stderr, "              2  Read AMR and AMR/NS output processed earlier and calculate measures\n");
   fprintf(stderr, "              3  Process AMR and AMR/NS and delete output, calculcate measures\n");
   fprintf(stderr, "              4  Process AMR and AMR/NS and store output, calculate measures\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "  Report_File_Name: name of destination file for report (default: 'Results.txt')\n");
   fprintf(stderr, "\nIf run without arguments or with first argument as '-h', this help is printed\n");
   fprintf(stderr, "\n");
   exit(1);

}



/*---------------------------------------------------------------------------
 
        char *NextFile(FileDesc *desc)
              ========

        Description:
        ~~~~~~~~~~~~

        Return next file information in the list claimed in DescDb.c, either by
        the next filename returning within the group of preprocessed file
        descriptions or by moving to the next group of preprocessed file descriptors.

        Arguments:
        ~~~~~~~~~~
        Name:        I/0: Type:    Use:
        desc          I   FileDesc File descriptor, or table of information records
                                   of sample files in database.

        Return value:
        ~~~~~~~~~~~~~
        Returns next file description in the descriptor structure, or zero if
        no file is found.

        Functions used:        none
        ~~~~~~~~~~~~~~~

        Prototype:             in main.c (this file)
        ~~~~~~~~~~

        Author:                Beghdad Ayad     Nokia Research Center   Finland
        ~~~~~~~                Erkki Paajanen   Nokia Research Center   Finland
                               <erkki.paajanen@nokia.com>

        Modification history:
        ~~~~~~~~~~~~~~~~~~~~~
        Date         Version   Description
        07/Jun/00    0.10      Preliminary version for review in SMG11
                               <erkki.paajanen@nokia.com>
        19/Jan/00    1.00      CR release for TS 06.77.
                               <erkki.paajanen@nokia.com>

 *--------------------------------------------------------------------------*/
/*
** Copyright (c) 1999-2001 Nokia Corporation
*/
char *NextFile(FileDesc *desc)
{
    static FileDesc *fileDesc = 0;
    static int currentCount = 0;
    static char name[500];
    char nametmp[500];
    int count;
    int g,id,s;

    if(desc != 0) {
        fileDesc = desc;
        currentCount = 0;
    }
    assert(fileDesc);

    count = 0;
    for(g = 0; g < 2; g++) { /* gender */
        for(id = 1; id <= 2; id++) { /* id */
            for(s = fileDesc->firstSample; s <= fileDesc->lastSample; s++) { /* sample */
                /* Seek filename number */
                if(count == currentCount) {
                    /* Construct the filename */
                    strcpy(name, fileDesc->base);
#if defined(__unix) || defined(__unix__)
                    strcat(name, "/exp");
                    strcat(name, fileDesc->expNum);
                    strcat(name, "/lab");
                    strcat(name, fileDesc->testLabAcronym);
                    strcat(name, "/ns");
                    strcat(name, fileDesc->candidateAcronym);
                    strcat(name, "/");
#else /* Use PC path name separators */
                    strcat(name, "\\exp");
                    strcat(name, fileDesc->expNum);
                    strcat(name, "\\lab");
                    strcat(name, fileDesc->testLabAcronym);
                    strcat(name, "\\ns");
                    strcat(name, fileDesc->candidateAcronym);
                    strcat(name, "\\");
#endif /* __unix, __unix__ */
                    strcat(name, gender[g]);
                    sprintf(nametmp, "%s%d", name, id);
                    strcat(nametmp, fileDesc->sampleBegString);
                    sprintf(name, "%s%*d", nametmp, fileDesc->sampleDigitNumber, s);
                    strcat(name, "c");
                    strcat(name, fileDesc->cond);
                    currentCount++;
                    return(name);
                } /* if count */
                count++;
            } /* for s */
        } /* for id */
    } /* for g */
    return(0);
}


